@@ -1,4 +1,4 @@
-/* $Id: process_xml.c,v 1.47 2006/03/23 08:39:56 knobi1 Exp $ */
+/* $Id: process_xml.c 670 2006-07-21 08:59:40Z knobi1 $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -516,7 +516,7 @@ startElement_HOST(void *data, const char *el, const char **attr)
    host->stringslen = edge;
 
    /* Trim structure to the correct length. */
-   hashval.size = sizeof(*host) - FRAMESIZE + host->stringslen;
+   hashval.size = sizeof(*host) - GMETAD_FRAMESIZE + host->stringslen;
    hashval.data = host;
 
    /* We dont care if this is an insert or an update. */
@@ -642,7 +642,7 @@ startElement_METRIC(void *data, const char *el, const char **attr)
          metric->t0.tv_sec -= metric->tn;
 
          /* Trim metric structure to the correct length. */
-         hashval.size = sizeof(*metric) - FRAMESIZE + metric->stringslen;
+         hashval.size = sizeof(*metric) - GMETAD_FRAMESIZE + metric->stringslen;
          hashval.data = (void*) metric;
 
          /* Update full metric in cluster host table. */
@@ -665,8 +665,6 @@ startElement_METRIC(void *data, const char *el, const char **attr)
                      metric = &(xmldata->metric);
                      memset((void*) metric, 0, sizeof(*metric));
                      fillmetric(attr, metric, type);
-                     metric->t0 = xmldata->now;
-                     metric->t0.tv_sec -= metric->tn;
                   }
                /* else we have already filled in the metric above. */
             }
@@ -696,9 +694,10 @@ startElement_METRIC(void *data, const char *el, const char **attr)
             }
 
          metric->num++;
+         metric->t0 = xmldata->now; /* tell cleanup thread we are using this */
 
          /* Trim metric structure to the correct length. Tricky. */
-         hashval.size = sizeof(*metric) - FRAMESIZE + metric->stringslen;
+         hashval.size = sizeof(*metric) - GMETAD_FRAMESIZE + metric->stringslen;
          hashval.data = (void*) metric;
 
          /* Update metric in summary table. */
@@ -792,7 +791,7 @@ startElement_METRICS(void *data, const char *el, const char **attr)
          }
 
    /* Update metric in summary table. */
-   hashval.size = sizeof(*metric) - FRAMESIZE + metric->stringslen;
+   hashval.size = sizeof(*metric) - GMETAD_FRAMESIZE + metric->stringslen;
    hashval.data = (void*) metric;
 
    summary = xmldata->source.metric_summary;
@@ -981,10 +980,11 @@ endElement_CLUSTER(void *data, const char *el)
 
          hashval.data = source;
          /* Trim structure to the correct length. */
-         hashval.size = sizeof(*source) - FRAMESIZE + source->stringslen;
+         hashval.size = sizeof(*source) - GMETAD_FRAMESIZE + source->stringslen;
 
          /* We insert here to get an accurate hosts up/down value. */
          rdatum = hash_insert( &hashkey, &hashval, xmldata->root);
+         source->sum_finished = NULL; /* remember that we released the lock */
          if (!rdatum) {
                err_msg("Could not insert source %s", xmldata->sourcename);
                return 1;
